#!/bin/bash
# MarmosetMuncher
# Submits files to Marmoset [ http://www.cs.umd.edu/users/jspacco/marmoset/ ]

# Course specific info
campusUID="$USER"
semester="Spring 2007"
courseName="CMSC420"
projectNumber="2"
archiveName="submit.zip"
submitFiles="*.java"

# Marmoset Submit Server URLs
oneTimePasswordURL='https://submit.cs.umd.edu:8443/eclipse/NegotiateOneTimePassword'
submitURL='https://submit.cs.umd.edu:8443/eclipse/SubmitProjectViaEclipse'
submitClientTool=MarmosetMuncher
submitClientVersion="6.6.6a"


# URL encode a stream or a string 
function url_encode() {
    # usage exit for too many parameters
    [ $# -gt 1 ] && { echo >&2 "usage: url_encode [string]"; return 1; }
    
    # self call when an argument is given, else handle stdin
    [ $# -eq 1 ] && { echo -n "$1" | url_encode; return $?; }
        
    # first create alternating lines of hex code and ascii characters
    # then remember the hex value,
    # convert spaces to +,
    # keep some selected characters unchanged
    # and use the hexvalue prefixed with a % for the rest
    hexdump -v -e '1/1 "%02x\t"' -e '1/1 "%_c\n"' |
    LANG=C awk '
        $1 == "20"                      { printf("%s",      "+");   next    }
        $2 ~  /^[a-zA-Z0-9.*()\/-]$/    { printf("%s",      $2);    next    }
                                        { printf("%%%s",    $1)             }
    '
}	


# Create Zip file of all files matching $submitFiles in CWD
echo "Creating Submission Archive"
echo
zip -r "$archiveName" . -i "$submitFiles" || exit 1

# Get password securely
echo
stty -echo
read -ers -p "UMD Directory Password for $campusUID: " uidPassword
stty echo
echo
echo

# Get a Marmoset "One Time Password"
formParams=" -d campusUID="`url_encode "$campusUID"`\
" -d uidPassword="`url_encode "$uidPassword"`\
" -d courseName="`url_encode "$courseName"`\
" -d semester="`url_encode "$semester"`\
" -d projectNumber="`url_encode "$projectNumber"` 

echo "Getting Marmoset \"One-Time Password...\""
response=`curl -s --insecure $formParams "$oneTimePasswordURL" | tr '\0\r\n' ' '`
cvsAccount=`expr match "$response" '.*cvsAccount=\([A-Za-z0-9]*\).*'`
oneTimePassword=`expr match "$response" '.*oneTimePassword=\([A-Za-z0-9]*\).*'`

# Submit project
echo "Submitting $projectNumber for $courseNaame $semester for $campusUID"
curl --insecure \
	-F "cvsAccount=$cvsAccount" \
	-F "oneTimePassword=$oneTimePassword" \
	-F "projectNumber=$projectNumber" \
	-F "semester=$semester" \
	-F "courseName=$courseName" \
	-F "submitClientTool=$submitClientTool" \
	-F "submitClientVersion=$submitClientVersion" \
	-F "submittedFiles=@$archiveName" \
	"$submitURL"
